@@ -524,7 +524,7 @@ hyperg_U_infinite_sum_simple(int N, double a, double bint, double b, double beps
         t_err = a0_err + b0_err;
         dchu_val += t_val;
         dchu_err += t_err;
-        if(!gsl_finite(t_val) || fabs(t_val) < EPS*fabs(dchu_val)) break;
+        if(!finite(t_val) || fabs(t_val) < EPS*fabs(dchu_val)) break;
       }
 
       result->val  = dchu_val;
@@ -680,6 +680,18 @@ int
 hyperg_U_series(const double a, const double b, const double x, gsl_sf_result * result)
 {
   const double SQRT_EPS = M_SQRT2 * GSL_SQRT_DBL_EPSILON;
+double bint = ( b < 0.0 ? ceil(b-0.5) : floor(b+0.5) );
+  double beps  = b - bint;
+  double a_beps = a - beps;
+  double r_a_beps = floor(a_beps + 0.5);
+  double a_beps_int = ( fabs(a_beps - r_a_beps) < INT_THRESHOLD );
+//  double a_b_1 = a-b+1;
+//  double r_a_b_1 = floor(a_b_1+0.5);
+//  double r_a_b_1_int = (fabs(a_b_1-r_a_b_1)< INT_THRESHOLD);
+  // Check for (a-beps) being a member of -N; N being 0,1,...
+  if (a_beps_int && a_beps <= 0)
+  { 	beps=beps - 1 + floor(a_beps);bint=bint + 1 - floor(a_beps);
+  }
 
   if(fabs(1.0 + a - b) < SQRT_EPS) {
     /* Original Comment: ALGORITHM IS BAD WHEN 1+A-B IS NEAR ZERO FOR SMALL X
@@ -694,9 +706,7 @@ hyperg_U_series(const double a, const double b, const double x, gsl_sf_result *
     return stat_e;
   }
   else {
-    double bint = ( b < 0.0 ? ceil(b-0.5) : floor(b+0.5) );
-    double beps  = b - bint;
-    int N = bint;
+     int N = bint;
     
     double lnx  = log(x);
     double xeps = exp(-beps*lnx);
@@ -711,7 +721,7 @@ hyperg_U_series(const double a, const double b, const double x, gsl_sf_result *
     if(fabs(xeps-1.0) > 0.5 ) {
       stat_inf = hyperg_U_infinite_sum_stable(N, a, bint, b, beps, x, xeps, sum, result);
     } else if (1+a-b < 0 && 1+a-b==floor(1+a-b) && beps != 0) {
-      stat_inf = hyperg_U_infinite_sum_simple(N, a, bint, b, beps, x, xeps, sum, result);
+       stat_inf = hyperg_U_infinite_sum_simple(N, a, bint, b, beps, x, xeps, sum, result);
     } else {
       stat_inf = hyperg_U_infinite_sum_improved(N, a, bint, b, beps, x, xeps, sum, result);
     }
@@ -1611,7 +1621,24 @@ hyperg_U_negx (const double a, const double b, const double x, gsl_sf_result_e10
 static int
 hyperg_U_int_negx (const int a, const int b, const double x, gsl_sf_result_e10 * result)
 {
-  return hyperg_U_negx (a, b, x, result);
+/* Looking at the tests it seems that everything is handled correctly by hyperg_U_negx 
+except a<b<=0.  The b=1 case seems strange since the poch(1+a-b,-a) should blow up. 
+In order to do no (little) harm I fix up only a<b<=0 using DLMF 13.2.11.  These are the failing conditions
+*/
+	if (a<b && b<=0) 
+	{	gsl_sf_result_e10 r1;
+		double	z21_z=pow(x, 1-b);
+		int	status = hyperg_U_negx (1+a-b,2-b, x, &r1);
+		double	res_tem=z21_z*r1.val;
+			result->val = res_tem;
+		double	res_tem_err = fabs(z21_z)*r1.err;
+			result->err = res_tem_err;
+	return status;
+	}
+	else {
+	  return hyperg_U_negx (a, b, x, result);
+	}
+	
 }  
 
 
@@ -1629,7 +1656,7 @@ gsl_sf_hyperg_U_int_e10_e(const int a, const int b, const double x,
   }
   else if (x == 0.0) {
     return hyperg_U_int_origin (a, b, result);
-  } else if (x < 0.0) {
+  } else if (x < 0.0)  {
     return hyperg_U_int_negx (a, b, x, result);
   }
   else {
@@ -1677,13 +1704,26 @@ gsl_sf_hyperg_U_e10_e(const double a, const double b, const double x,
     result->err = 0.0;
     result->e10 = 0;
     return GSL_SUCCESS;
-  } else if (x == 0.0) {
+  } 
+  else if (x == 0.0) {
     return hyperg_U_origin (a, b, result);
-  } else if (x < 0.0) {
-    return hyperg_U_negx (a, b, x, result);
+  } 
+  else if(a_integer && b == a + 1)
+/* This is DLMF 13.6.4 */
+   {
+     gsl_sf_result powx1N_1;
+     int stat_p_1 = gsl_sf_pow_int_e(x, -a, &powx1N_1);
+     result->val = powx1N_1.val;
+     result->err = powx1N_1.err;
+     result->e10 = 0;
+     return GSL_SUCCESS;
+	
+   }  
+   else if(a_integer && b_integer) {
+     return gsl_sf_hyperg_U_int_e10_e(rinta, rintb, x, result);
   }
-  else if(a_integer && b_integer) {
-    return gsl_sf_hyperg_U_int_e10_e(rinta, rintb, x, result);
+  else if (x < 0.0) {
+    return hyperg_U_negx (a, b, x, result);
   }
   else {
     if(b >= 1.0) {
